/*
/--------------------------------------------------------------------
|
|      $Id: FilterContrast.cpp,v 1.1 2000/11/06 23:20:22 uzadow Exp $
|
|      Copyright (c) 1996-1998 Ulrich von Zadow
|
\--------------------------------------------------------------------
*/

#include "stdpch.h"
#include "FilterContrast.h"
#include "bitmap.h"
#include "anybmp.h"

CFilterContrast::CFilterContrast(double contrast, BYTE offset)
{
  m_contrast = contrast;
  m_offset   = (double) offset-125.0;
}

CFilterContrast::~CFilterContrast()
{

}

void CFilterContrast::Apply(CBmp * pBmpSource, CBmp * pBmpDest) 
{
  INT total = 0;
  register int inc;

  double contrast = m_contrast;
 
  PLASSERT (pBmpSource->GetBitsPerPixel() >= 24);

  pBmpDest->Create (pBmpSource->GetWidth(), 
                    pBmpSource->GetHeight(),
                    pBmpSource->GetBitsPerPixel(),
                    pBmpSource->HasAlpha());

  BYTE ** pSrcLines = pBmpSource->GetLineArray();
  BYTE ** pDstLines = pBmpDest->GetLineArray();

  register int destWidth = pBmpDest->GetWidth();

  if(pBmpSource->GetBitsPerPixel() == 24)
   inc = 3;

  if(pBmpSource->GetBitsPerPixel() == 32)
   inc = sizeof(RGBAPIXEL);

  register double red, green, blue;
  register double csupp = contrast * (m_offset - 125.0) + 125.0;

  for (int y = 0; y < pBmpDest->GetHeight(); ++y)
  { // For each line
    register BYTE * pSrcPixel = pSrcLines[y];
    register BYTE * pDstPixel = pDstLines[y];

    for (register int x = 0; x < destWidth; ++x)
    {
       // Formel fr Kontrastberechnung:
       // v = (contrast * (v - 125.0 + m_offset) + 125.0);
       red   = contrast * ((double) (pSrcPixel[RGBA_RED])) + csupp;
       green = contrast * ((double) (pSrcPixel[RGBA_GREEN])) + csupp;
       blue  = contrast * ((double) (pSrcPixel[RGBA_BLUE])) + csupp;

       if(red >= 255.0)
        pDstPixel[RGBA_RED] = (BYTE) 255;
       else if (red < 0.0)
        pDstPixel[RGBA_RED] = (BYTE) 0;
       else
        pDstPixel[RGBA_RED] = (BYTE) red;

       if(green >= 255.0)
        pDstPixel[RGBA_GREEN] = (BYTE) 255;
       else if (green < 0.0)
        pDstPixel[RGBA_GREEN] = (BYTE) 0;
       else
        pDstPixel[RGBA_GREEN] = (BYTE) green;

       if(blue >= 255.0)
        pDstPixel[RGBA_BLUE] = (BYTE) 255;
       else if (blue < 0.0)
        pDstPixel[RGBA_BLUE] = (BYTE) 0;
       else
        pDstPixel[RGBA_BLUE] = (BYTE) blue;

       pSrcPixel += inc;
       pDstPixel += inc;
    }
  }
}

/*
/--------------------------------------------------------------------
|
|      $Log: FilterContrast.cpp,v $
|      Revision 1.1  2000/11/06 23:20:22  uzadow
|      Added Contrast, Intensity and Lightness filters by
|      Thomas Hirschmann
|
|
\--------------------------------------------------------------------
*/
