/*
/--------------------------------------------------------------------
|
|      $Id: FilterQuantize.h,v 1.1 2000/09/26 12:14:49 Administrator Exp $
|
|      Copyright (c) 1996-1998 Ulrich von Zadow
|
\--------------------------------------------------------------------
*/

#if !defined(INCL_FILTERQUANTIZE)
#define INCL_FILTERQUANTIZE

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "Filter.h"
#include "bitmap.h"

#define DTHPAL_MEDIAN          0       // Median Cut
#define DTHPAL_POPULARITY      1       // Popularity Sort
#define DTHPAL_DEFAULT         2       // Use Default Palette
//#define DTHPAL_NEUQUANTBEST   3		// Neural Quantization Best/Slowest Quality
//#define DTHPAL_NEUQUANTMIDDLE	4		// Neural Quantization Middle/average Speed Quality
//#define DTHPAL_NEUQUANTWORST	5		// Neural Quantization Worst/Fastest Quality

#define DTH_NONE      0       // None
#define DTH_ORDERED   1       // Ordered Dithering
#define DTH_FS				2       // Floyd-Steinberg Dithering
//#define DTH_JITTER    3       // Jitter preprocessing


//! Returns an 8 bpp bitmap containing a best-fit representation of the
//! source 32 bpp bitmap.
class CFilterQuantize : public CFilter
{
public:
  //! DitherPaletteType and DitherType determine the type of quantization:
  //!
  //! DTHPAL_MEDIAN (0): Median Cut
  //!
  //! DTHPAL_POPULARITY (1): Popularity Sort
  //!
  //! DTHPAL_DEFAULT (2): Use Default Palette
  //!
  //!
  //! DTH_NONE (0): None
  //!
  //! DTH_ORDERED (1): Ordered Dithering
  //!
  //! DTH_FS (2): Floyd-Steinberg Dithering
	CFilterQuantize (int DitherPaletteType, int DitherType);
  //!
	virtual ~CFilterQuantize();

  //! 
  virtual void Apply(CBmp * pBmpSource, CBmp * pBmpDest);

private:
  // Local structs defining the quantization buffer
  typedef struct tagQUBOX
  {
	  RGBAPIXEL Corner0, Corner1;
    RGBAPIXEL Average;
	  ULONG count;
  } QUBOX;

  typedef struct tagHISTONODE
  {
	  int		index;
	  ULONG	count; 
  } HISTONODE;

  // Private members
  void initLUT();
  void deleteLUT();

  void genMedianPalette (CBmp * pBmpSource, CBmp * pBmpDest);
  void split (QUBOX * pBox0, QUBOX * pBox1, int ColComp);
  void squeeze(QUBOX * pBox);
  void genPopularityPalette (CBmp * pBmpSource, CBmp * pBmpDest);
  void genColorArray(CBmp * pBmpSource);
  void genDefaultPalette (CBmp * pBmpSource);
  void addColor(RGBAPIXEL col, ULONG count);
  void makeBox(RGBAPIXEL col, int i, ULONG c);
  int getColorTableIndex (RGBAPIXEL col);
  int getShiftedColorTableIndex (RGBAPIXEL col);

  void ditherDestBmp(CBmp * pBmpSource, CBmp * pBmpDest);
  void jitterPixel (int i, int y, RGBAPIXEL * pPixel);
  void ditherPixelOrdered (int x, int y, RGBAPIXEL * pPixel);
  void ditherCompOrdered (int x, int y, BYTE * pComp);
  void ditherPixelFS(double * pR, double * pG, double * pB, double * pCurErrors);
  void ditherCompFS (double * pComp, double Error);
  BYTE getNeighbor (RGBAPIXEL Color, RGBAPIXEL * pPal);
  int colorDist (RGBAPIXEL c0, RGBAPIXEL c1);

  int clip (int c);

  int m_DitherPaletteType;
  int m_DitherType;

  HISTONODE ** m_ppHisto; // Array of 32x32x32 color bins. Allocated
                          // sparsely to save memory.
  QUBOX * m_pQuBoxes;     // Array of 256 boxes, one for each color.
};

inline int CFilterQuantize::clip (int c)
{
  if (c > 255)
    return 255;
  if (c < 0)
    return 0;
  return c;
}

#endif

/*
/--------------------------------------------------------------------
|
|      $Log: FilterQuantize.h,v $
|      Revision 1.1  2000/09/26 12:14:49  Administrator
|      Refactored quantization.
|
|
|
\--------------------------------------------------------------------
*/
