/*
/--------------------------------------------------------------------
|
|      Windows bitmap file encoder. Encodes 1, 4, 8, 24 and 32 bpp
|      bitmaps to a 1, 4, 8 or 24 bpp uncompressed BMP file
|
|      Copyright (c) 1996-1998 Ulrich von Zadow
|
\--------------------------------------------------------------------
*/
#include "stdpch.h"
#include "bmpenc.h"
#include "windefs.h"
#include "except.h"


/////////////////////////////////////////////////////////////////////
// Class functions

CBmpEncoder::CBmpEncoder
    ()
    // Creates an encoder
{
  // nothing to do (yet)
}


CBmpEncoder::~CBmpEncoder
    ()
{
  // nothing to do (yet)
}

int CBmpEncoder::GetLineMemNeeded (LONG width, WORD BitsPerPixel)
{
  // Calculate memory per line.
  int LineMem = width * BitsPerPixel / 8;

  if (BitsPerPixel == 1 && width % 8)
    ++LineMem;

  // DWORD alignment
  return ((LineMem + 3) & ~3);
}


void CBmpEncoder::DoEncode
    ( CBmp * pBmp,
      CDataSink * pSink
    )
{
  int BPP = pBmp->GetBitsPerPixel();

  WINBITMAPFILEHEADER BFH;
  BFH.bfType = *((WORD*)"BM");
  BFH.bfReserved1 = 0;
  BFH.bfReserved2 = 0;
  BFH.bfOffBits = sizeof (WINBITMAPFILEHEADER) +
                  sizeof (WINBITMAPINFOHEADER);
  if (BPP <= 8)   // include palette
    BFH.bfOffBits += (1 << BPP) * sizeof(WINRGBQUAD);

  BFH.bfSize = BFH.bfOffBits;
  BFH.bfSize += pBmp->GetHeight()*GetLineMemNeeded(pBmp->GetWidth(), BPP);

  pSink->WriteNBytes( sizeof (WINBITMAPFILEHEADER), (BYTE *) &BFH);

  WINBITMAPINFOHEADER FileBMI;
  FileBMI.biSize = sizeof(WINBITMAPINFOHEADER);
  FileBMI.biWidth = pBmp->GetWidth ();
  FileBMI.biHeight = pBmp->GetHeight ();
  FileBMI.biPlanes = 1;
  FileBMI.biCompression = BI_RGB;
  FileBMI.biSizeImage = 0; // may be 0 for non-compressed files
  FileBMI.biXPelsPerMeter = 0;
  FileBMI.biYPelsPerMeter = 0;
  FileBMI.biClrUsed = 0;
  FileBMI.biClrImportant = 0;

  BYTE * pCurLine;
  int x,y;
  if (BPP <= 8)
  {
    FileBMI.biBitCount = BPP;  // not 32...
    pSink->WriteNBytes (sizeof (WINBITMAPINFOHEADER), (BYTE *) &FileBMI);
    pSink->WriteNBytes ((1 << BPP) * sizeof(WINRGBQUAD), (BYTE *) pBmp->GetPalette());

    for (y=FileBMI.biHeight-1; y>=0; y--)
    {
      pCurLine = pBmp->GetLineArray()[y];
      pSink->WriteNBytes (pBmp->GetBytesPerLine(), pCurLine);
      int PadBytes = GetLineMemNeeded(pBmp->GetWidth(), BPP) 
        - pBmp->GetBytesPerLine();

      pSink->Skip(PadBytes);
    }
  }
  else
  {
    FileBMI.biBitCount = 24;  // not 32...
    pSink->WriteNBytes (sizeof (WINBITMAPINFOHEADER),(BYTE *) &FileBMI);

    int LinePadding = 4-((FileBMI.biWidth*3)&3);
    if (LinePadding == 4)
      LinePadding = 0;
    for (y=FileBMI.biHeight-1; y>=0; y--)
    {
      pCurLine = pBmp->GetLineArray()[y];
      for (x=0; x<FileBMI.biWidth; x++)
      {
#ifdef _WINDOWS
        pSink->WriteNBytes(3, pCurLine+x*4);
#else 
        pSink->WriteNBytes(1, pCurLine+x*4+RGBA_BLUE);
        pSink->WriteNBytes(1, pCurLine+x*4+RGBA_GREEN);
        pSink->WriteNBytes(1, pCurLine+x*4+RGBA_RED);
#endif
      }
      pSink->WriteNBytes(LinePadding,(BYTE *) " ");
    }
  }

}

/*
/--------------------------------------------------------------------
|
|      $Log: bmpenc.cpp,v $
|      Revision 1.7  2000/12/02 19:52:12  uzadow
|      no message
|
|      Revision 1.6  2000/12/02 19:50:01  uzadow
|      Added Logging.
|
|    31.10.2000 - Jos Miguel Buenaposada
|                 Corrected the order for writing color channels
|                 to work with other channels orders.
|
|    27.08.2000 - Martin Skinner
|                 now uses CDataSink instead of MFC CFile
|                 moved to a new class CBmpEncoder
|
|    11.01.2000 - Michael Salzlechner
|                 moved code from CWinBmp to CAfxBmp
|
|    21.01.99 - Added by M.Skinner
|               Moved program logic from 
|               CWinBmp::SaveAsBmp(const char *) to here
|
\--------------------------------------------------------------------
*/
